@@ -77,6 +77,11 @@ class ProgressBarBase(Callback):
     def predict_description(self) -> str:
         return "Predicting"
 
+    @property
+    def _val_processed(self) -> int:
+        # use total in case validation runs more than once per training epoch
+        return self.trainer.fit_loop.epoch_loop.val_loop.epoch_loop.batch_progress.total.processed
+
     @property
     def train_batch_idx(self) -> int:
         """The number of batches processed during training.
